@@ -82,16 +82,12 @@ void cgroup_exit(struct cgroup_ops *ops)
 	free(ops->container_cgroup);
 
 	for (it = ops->hierarchies; it && *it; it++) {
-		char **p;
+		char **ctrlr;
 
-		for (p = (*it)->controllers; p && *p; p++)
-			free(*p);
+		for (ctrlr = (*it)->controllers; ctrlr && *ctrlr; ctrlr++)
+			free(*ctrlr);
 		free((*it)->controllers);
 
-		for (p = (*it)->cgroup2_chown; p && *p; p++)
-			free(*p);
-		free((*it)->cgroup2_chown);
-
 		free((*it)->mountpoint);
 		free((*it)->container_base_path);
 		free((*it)->container_full_path);
@@ -100,8 +96,6 @@ void cgroup_exit(struct cgroup_ops *ops)
 	}
 	free(ops->hierarchies);
 
-	free(ops);
-
 	return;
 }
 
