@@ -51,13 +51,10 @@ static int do_freeze_thaw(bool freeze, struct lxc_conf *conf, const char *name,
 	int ret;
 	char v[100];
 	struct cgroup_ops *cgroup_ops;
-	const char *state;
-	size_t state_len;
+	const char *state = freeze ? "FROZEN" : "THAWED";
+	size_t state_len = 6;
 	lxc_state_t new_state = freeze ? FROZEN : THAWED;
 
-	state = lxc_state2str(new_state);
-	state_len = strlen(state);
-
 	cgroup_ops = cgroup_init(conf);
 	if (!cgroup_ops)
 		return -1;
@@ -65,21 +62,19 @@ static int do_freeze_thaw(bool freeze, struct lxc_conf *conf, const char *name,
 	ret = cgroup_ops->set(cgroup_ops, "freezer.state", state, name, lxcpath);
 	if (ret < 0) {
 		cgroup_exit(cgroup_ops);
-		ERROR("Failed to %s %s",
-		      (new_state == FROZEN ? "freeze" : "unfreeze"), name);
+		ERROR("Failed to freeze %s", name);
 		return -1;
 	}
 
 	for (;;) {
-		ret = cgroup_ops->get(cgroup_ops, "freezer.state", v, sizeof(v),
-				      name, lxcpath);
+		ret = cgroup_ops->get(cgroup_ops, "freezer.state", v, sizeof(v), name, lxcpath);
 		if (ret < 0) {
 			cgroup_exit(cgroup_ops);
 			ERROR("Failed to get freezer state of %s", name);
 			return -1;
 		}
 
-		v[sizeof(v) - 1] = '\0';
+		v[99] = '\0';
 		v[lxc_char_right_gc(v, strlen(v))] = '\0';
 
 		ret = strncmp(v, state, state_len);
