@@ -26,6 +26,7 @@
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE 1
 #endif
+#include <alloca.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -66,7 +67,6 @@
 #include "lxcseccomp.h"
 #include "macro.h"
 #include "mainloop.h"
-#include "memory_utils.h"
 #include "monitor.h"
 #include "namespace.h"
 #include "network.h"
@@ -97,13 +97,16 @@ static void lxc_destroy_container_on_signal(struct lxc_handler *handler,
 
 static void print_top_failing_dir(const char *path)
 {
-	__do_free char *copy = NULL;
 	int ret;
-	char *e, *p, saved;
+	size_t len;
+	char *copy, *e, *p, saved;
+
+	len = strlen(path);
+	copy = alloca(len + 1);
+	(void)strlcpy(copy, path, len + 1);
 
-	copy = must_copy_string(path);
 	p = copy;
-	e = copy + strlen(path);
+	e = copy + len;
 
 	while (p < e) {
 		while (p < e && *p == '/')
@@ -406,26 +409,14 @@ static int signal_handler(int fd, uint32_t events, void *data,
 	}
 
 	if (siginfo.ssi_signo == SIGHUP) {
-		if (hdlr->pidfd >= 0)
-			lxc_raw_pidfd_send_signal(hdlr->pidfd, SIGTERM, NULL, 0);
-		else if (hdlr->proc_pidfd >= 0)
-			lxc_raw_pidfd_send_signal(hdlr->proc_pidfd, SIGTERM, NULL, 0);
-		else
-			kill(hdlr->pid, SIGTERM);
+		kill(hdlr->pid, SIGTERM);
 		INFO("Killing %d since terminal hung up", hdlr->pid);
 		return hdlr->init_died ? LXC_MAINLOOP_CLOSE
 				       : LXC_MAINLOOP_CONTINUE;
 	}
 
 	if (siginfo.ssi_signo != SIGCHLD) {
-		if (hdlr->pidfd >= 0)
-			lxc_raw_pidfd_send_signal(hdlr->pidfd,
-						  siginfo.ssi_signo, NULL, 0);
-		else if (hdlr->proc_pidfd >= 0)
-			lxc_raw_pidfd_send_signal(hdlr->proc_pidfd,
-						  siginfo.ssi_signo, NULL, 0);
-		else
-			kill(hdlr->pid, siginfo.ssi_signo);
+		kill(hdlr->pid, siginfo.ssi_signo);
 		INFO("Forwarded signal %d to pid %d", siginfo.ssi_signo, hdlr->pid);
 		return hdlr->init_died ? LXC_MAINLOOP_CLOSE
 				       : LXC_MAINLOOP_CONTINUE;
@@ -464,6 +455,7 @@ int lxc_serve_state_clients(const char *name, struct lxc_handler *handler,
 	size_t retlen;
 	ssize_t ret;
 	struct lxc_list *cur, *next;
+	struct lxc_state_client *client;
 	struct lxc_msg msg = {.type = lxc_msg_state, .value = state};
 
 	if (state == THAWED)
@@ -483,7 +475,7 @@ int lxc_serve_state_clients(const char *name, struct lxc_handler *handler,
 		return -E2BIG;
 
 	lxc_list_for_each_safe(cur, &handler->conf->state_clients, next) {
-		struct lxc_state_client *client = cur->elem;
+		client = cur->elem;
 
 		if (client->states[state] == 0) {
 			TRACE("State %s not registered for state client %d",
@@ -664,10 +656,6 @@ void lxc_zero_handler(struct lxc_handler *handler)
 
 	handler->pinfd = -1;
 
-	handler->pidfd = -EBADF;
-
-	handler->proc_pidfd = -EBADF;
-
 	handler->sigfd = -1;
 
 	for (i = 0; i < LXC_NS_MAX; i++)
@@ -688,12 +676,6 @@ void lxc_free_handler(struct lxc_handler *handler)
 	if (handler->pinfd >= 0)
 		close(handler->pinfd);
 
-	if (handler->pidfd >= 0)
-		close(handler->pidfd);
-
-	if (handler->proc_pidfd >= 0)
-		close(handler->proc_pidfd);
-
 	if (handler->sigfd >= 0)
 		close(handler->sigfd);
 
@@ -738,8 +720,6 @@ struct lxc_handler *lxc_init_handler(const char *name, struct lxc_conf *conf,
 	handler->conf = conf;
 	handler->lxcpath = lxcpath;
 	handler->pinfd = -1;
-	handler->pidfd = -EBADF;
-	handler->proc_pidfd = -EBADF;
 	handler->sigfd = -EBADF;
 	handler->init_died = false;
 	handler->state_socket_pair[0] = handler->state_socket_pair[1] = -1;
@@ -916,9 +896,20 @@ int lxc_init(const char *name, struct lxc_handler *handler)
 	}
 	TRACE("Initialized cgroup driver");
 
+	ret = lsm_process_prepare(conf, handler->lxcpath);
+	if (ret < 0) {
+		ERROR("Failed to initialize LSM");
+		goto out_destroy_cgroups;
+	}
+	TRACE("Initialized LSM");
+
 	INFO("Container \"%s\" is initialized", name);
 	return 0;
 
+out_destroy_cgroups:
+	handler->cgroup_ops->payload_destroy(handler->cgroup_ops, handler);
+	handler->cgroup_ops->monitor_destroy(handler->cgroup_ops, handler);
+
 out_delete_terminal:
 	lxc_terminal_delete(&handler->conf->console);
 
@@ -1009,6 +1000,8 @@ void lxc_fini(const char *name, struct lxc_handler *handler)
 	while (namespace_count--)
 		free(namespaces[namespace_count]);
 
+	lsm_process_cleanup(handler->conf, handler->lxcpath);
+
 	cgroup_ops->payload_destroy(cgroup_ops, handler);
 	cgroup_ops->monitor_destroy(cgroup_ops, handler);
 
@@ -1037,9 +1030,6 @@ void lxc_fini(const char *name, struct lxc_handler *handler)
 		lxc_set_state(name, handler, STOPPED);
 	}
 
-	/* Avoid lingering namespace references. */
-	lxc_put_nsfds(handler);
-
 	ret = run_lxc_hooks(name, "post-stop", handler->conf, NULL);
 	if (ret < 0) {
 		ERROR("Failed to run lxc.hook.post-stop for container \"%s\"", name);
@@ -1088,30 +1078,23 @@ void lxc_fini(const char *name, struct lxc_handler *handler)
 
 void lxc_abort(const char *name, struct lxc_handler *handler)
 {
-	int ret = 0;
-	int status;
+	int ret, status;
 
 	lxc_set_state(name, handler, ABORTING);
 
-	if (handler->pidfd > 0)
-		ret = lxc_raw_pidfd_send_signal(handler->pidfd, SIGKILL, NULL, 0);
-	else if (handler->proc_pidfd > 0)
-		ret = lxc_raw_pidfd_send_signal(handler->proc_pidfd, SIGKILL, NULL, 0);
-	else if (handler->pid > 0)
+	if (handler->pid > 0) {
 		ret = kill(handler->pid, SIGKILL);
-	if (ret < 0)
-		SYSERROR("Failed to send SIGKILL to %d", handler->pid);
+		if (ret < 0)
+			SYSERROR("Failed to send SIGKILL to %d", handler->pid);
+	}
 
-	do {
-		ret = waitpid(-1, &status, 0);
-	} while (ret > 0);
+	while ((ret = waitpid(-1, &status, 0)) > 0) {
+		;
+	}
 }
 
 static int do_start(void *data)
 {
-	struct lxc_handler *handler = data;
-	ATTR_UNUSED __do_close_prot_errno int data_sock0 = handler->data_sock[0],
-					      data_sock1 = handler->data_sock[1];
 	int ret;
 	char path[PATH_MAX];
 	uid_t new_uid;
@@ -1120,6 +1103,7 @@ static int do_start(void *data)
 	uid_t nsuid = 0;
 	gid_t nsgid = 0;
 	int devnull_fd = -1;
+	struct lxc_handler *handler = data;
 
 	lxc_sync_fini_parent(handler);
 
@@ -1129,7 +1113,7 @@ static int do_start(void *data)
 	 * exit before we set the pdeath signal leading to a unsupervized
 	 * container.
 	 */
-	ret = lxc_set_death_signal(SIGKILL, handler->monitor_pid);
+	ret = lxc_set_death_signal(SIGKILL, 0);
 	if (ret < 0) {
 		SYSERROR("Failed to set PR_SET_PDEATHSIG to SIGKILL");
 		goto out_warn_father;
@@ -1207,7 +1191,7 @@ static int do_start(void *data)
 			goto out_warn_father;
 
 		/* set{g,u}id() clears deathsignal */
-		ret = lxc_set_death_signal(SIGKILL, handler->monitor_pid);
+		ret = lxc_set_death_signal(SIGKILL, 0);
 		if (ret < 0) {
 			SYSERROR("Failed to set PR_SET_PDEATHSIG to SIGKILL");
 			goto out_warn_father;
@@ -1268,16 +1252,10 @@ static int do_start(void *data)
 	if (handler->ns_clone_flags & CLONE_NEWCGROUP) {
 		ret = unshare(CLONE_NEWCGROUP);
 		if (ret < 0) {
-			if (errno != EINVAL) {
-				SYSERROR("Failed to unshare CLONE_NEWCGROUP");
-				goto out_warn_father;
-			}
-
-			handler->ns_clone_flags &= ~CLONE_NEWCGROUP;
-			SYSINFO("Kernel does not support CLONE_NEWCGROUP");
-		} else {
-			INFO("Unshared CLONE_NEWCGROUP");
+			INFO("Failed to unshare CLONE_NEWCGROUP");
+			goto out_warn_father;
 		}
+		INFO("Unshared CLONE_NEWCGROUP");
 	}
 
 	/* Add the requested environment variables to the current environment to
@@ -1295,13 +1273,15 @@ static int do_start(void *data)
 
 	/* Setup the container, ip, names, utsname, ... */
 	ret = lxc_setup(handler);
+	close(handler->data_sock[1]);
+	close(handler->data_sock[0]);
 	if (ret < 0) {
 		ERROR("Failed to setup container \"%s\"", handler->name);
 		goto out_warn_father;
 	}
 
 	/* Set the label to change to when we exec(2) the container's init. */
-	ret = lsm_process_label_set(NULL, handler->conf, 1, 1);
+	ret = lsm_process_label_set(NULL, handler->conf, true);
 	if (ret < 0)
 		goto out_warn_father;
 
@@ -1452,6 +1432,15 @@ static int do_start(void *data)
 		goto out_warn_father;
 	}
 
+	if (handler->conf->monitor_signal_pdeath != SIGKILL) {
+		ret = lxc_set_death_signal(handler->conf->monitor_signal_pdeath, 0);
+		if (ret < 0) {
+			SYSERROR("Failed to set PR_SET_PDEATHSIG to %d",
+				 handler->conf->monitor_signal_pdeath);
+			goto out_warn_father;
+		}
+	}
+
 	/* After this call, we are in error because this ops should not return
 	 * as it execs.
 	 */
@@ -1515,11 +1504,11 @@ int resolve_clone_flags(struct lxc_handler *handler)
 	struct lxc_conf *conf = handler->conf;
 
 	for (i = 0; i < LXC_NS_MAX; i++) {
-		if (conf->ns_keep) {
-			if (!(conf->ns_keep & ns_info[i].clone_flag))
+		if (conf->ns_keep > 0) {
+			if ((conf->ns_keep & ns_info[i].clone_flag) == 0)
 				handler->ns_clone_flags |= ns_info[i].clone_flag;
-		} else if (conf->ns_clone) {
-			if ((conf->ns_clone & ns_info[i].clone_flag))
+		} else if (conf->ns_clone > 0) {
+			if ((conf->ns_clone & ns_info[i].clone_flag) > 0)
 				handler->ns_clone_flags |= ns_info[i].clone_flag;
 		} else {
 			if (i == LXC_NS_USER && lxc_list_empty(&handler->conf->id_map))
@@ -1582,35 +1571,13 @@ static inline int do_share_ns(void *arg)
 
 	flags = handler->ns_on_clone_flags;
 	flags |= CLONE_PARENT;
-	handler->pid = lxc_raw_clone_cb(do_start, handler, CLONE_PIDFD | flags,
-					&handler->pidfd);
+	handler->pid = lxc_raw_clone_cb(do_start, handler, flags);
 	if (handler->pid < 0)
 		return -1;
 
 	return 0;
 }
 
-static int proc_pidfd_open(pid_t pid)
-{
-	__do_close_prot_errno int proc_pidfd = -EBADF;
-	char path[100];
-
-	snprintf(path, sizeof(path), "/proc/%d", pid);
-	proc_pidfd = open(path, O_DIRECTORY | O_RDONLY | O_CLOEXEC);
-	if (proc_pidfd < 0) {
-		SYSERROR("Failed to open %s", path);
-		return -1;
-	}
-
-	/* Test whether we can send signals. */
-	if (lxc_raw_pidfd_send_signal(proc_pidfd, 0, NULL, 0)) {
-		SYSERROR("Failed to send signal through pidfd");
-		return -1;
-	}
-
-	return move_fd(proc_pidfd);
-}
-
 /* lxc_spawn() performs crucial setup tasks and clone()s the new process which
  * exec()s the requested container binary.
  * Note that lxc_spawn() runs in the parent namespaces. Any operations performed
@@ -1620,7 +1587,6 @@ static int proc_pidfd_open(pid_t pid)
  */
 static int lxc_spawn(struct lxc_handler *handler)
 {
-	__do_close_prot_errno int data_sock0 = -EBADF, data_sock1 = -EBADF;
 	int i, ret;
 	char pidstr[20];
 	bool wants_to_map_ids;
@@ -1653,8 +1619,6 @@ static int lxc_spawn(struct lxc_handler *handler)
 			 handler->data_sock);
 	if (ret < 0)
 		goto out_sync_fini;
-	data_sock0 = handler->data_sock[0];
-	data_sock1 = handler->data_sock[1];
 
 	ret = resolve_clone_flags(handler);
 	if (ret < 0)
@@ -1717,7 +1681,7 @@ static int lxc_spawn(struct lxc_handler *handler)
 		pid_t attacher_pid;
 
 		attacher_pid = lxc_clone(do_share_ns, handler,
-					 CLONE_VFORK | CLONE_VM | CLONE_FILES, NULL);
+					 CLONE_VFORK | CLONE_VM | CLONE_FILES);
 		if (attacher_pid < 0) {
 			SYSERROR(LXC_CLONE_ERROR);
 			goto out_delete_net;
@@ -1730,8 +1694,7 @@ static int lxc_spawn(struct lxc_handler *handler)
 		}
 	} else {
 		handler->pid = lxc_raw_clone_cb(do_start, handler,
-						CLONE_PIDFD | handler->ns_on_clone_flags,
-						&handler->pidfd);
+						handler->ns_on_clone_flags);
 	}
 	if (handler->pid < 0) {
 		SYSERROR(LXC_CLONE_ERROR);
@@ -1739,12 +1702,6 @@ static int lxc_spawn(struct lxc_handler *handler)
 	}
 	TRACE("Cloned child process %d", handler->pid);
 
-	if (handler->pidfd < 0) {
-		handler->proc_pidfd = proc_pidfd_open(handler->pid);
-		if (handler->proc_pidfd < 0 && (errno != ENOSYS))
-			goto out_delete_net;
-	}
-
 	for (i = 0; i < LXC_NS_MAX; i++)
 		if (handler->ns_on_clone_flags & ns_info[i].clone_flag)
 			INFO("Cloned %s", ns_info[i].flag_name);
@@ -1792,24 +1749,23 @@ static int lxc_spawn(struct lxc_handler *handler)
 	if (!cgroup_ops->chown(cgroup_ops, handler->conf))
 		goto out_delete_net;
 
-	/* If not done yet, we're now ready to preserve the network namespace */
-	if (handler->nsfd[LXC_NS_NET] < 0) {
-		ret = lxc_try_preserve_ns(handler->pid, "net");
-		if (ret < 0) {
-			if (ret != -EOPNOTSUPP) {
-				SYSERROR("Failed to preserve net namespace");
-				goto out_delete_net;
-			}
-		} else {
-			handler->nsfd[LXC_NS_NET] = ret;
-			DEBUG("Preserved net namespace via fd %d", ret);
+	/* Now we're ready to preserve the network namespace */
+	ret = lxc_try_preserve_ns(handler->pid, "net");
+	if (ret < 0) {
+		if (ret != -EOPNOTSUPP) {
+			SYSERROR("Failed to preserve net namespace");
+			goto out_delete_net;
 		}
+	} else {
+		handler->nsfd[LXC_NS_NET] = ret;
+		DEBUG("Preserved net namespace via fd %d", ret);
+
+		ret = lxc_netns_set_nsid(handler->nsfd[LXC_NS_NET]);
+		if (ret < 0)
+			SYSWARN("Failed to allocate new network namespace id");
+		else
+			TRACE("Allocated new network namespace id");
 	}
-	ret = lxc_netns_set_nsid(handler->nsfd[LXC_NS_NET]);
-	if (ret < 0)
-		SYSWARN("Failed to allocate new network namespace id");
-	else
-		TRACE("Allocated new network namespace id");
 
 	/* Create the network configuration. */
 	if (handler->ns_clone_flags & CLONE_NEWNET) {
@@ -1969,7 +1925,7 @@ int __lxc_start(const char *name, struct lxc_handler *handler,
 	ret = lxc_init(name, handler);
 	if (ret < 0) {
 		ERROR("Failed to initialize container \"%s\"", name);
-		goto out_fini_nonet;
+		return -1;
 	}
 	handler->ops = ops;
 	handler->data = data;
@@ -1978,19 +1934,16 @@ int __lxc_start(const char *name, struct lxc_handler *handler,
 
 	if (!attach_block_device(handler->conf)) {
 		ERROR("Failed to attach block device");
-		ret = -1;
 		goto out_fini_nonet;
 	}
 
 	if (!cgroup_ops->monitor_create(cgroup_ops, handler)) {
 		ERROR("Failed to create monitor cgroup");
-		ret = -1;
 		goto out_fini_nonet;
 	}
 
 	if (!cgroup_ops->monitor_enter(cgroup_ops, handler->monitor_pid)) {
 		ERROR("Failed to enter monitor cgroup");
-		ret = -1;
 		goto out_fini_nonet;
 	}
 
@@ -2019,6 +1972,11 @@ int __lxc_start(const char *name, struct lxc_handler *handler,
 		ERROR("Failed to spawn container \"%s\"", name);
 		goto out_detach_blockdev;
 	}
+	/* close parent side of data socket */
+	close(handler->data_sock[0]);
+	handler->data_sock[0] = -1;
+	close(handler->data_sock[1]);
+	handler->data_sock[1] = -1;
 
 	handler->conf->reboot = REBOOT_NONE;
 
@@ -2030,7 +1988,6 @@ int __lxc_start(const char *name, struct lxc_handler *handler,
 
 	if (!handler->init_died && handler->pid > 0) {
 		ERROR("Child process is not killed");
-		ret = -1;
 		goto out_abort;
 	}
 
