@@ -44,28 +44,10 @@
 #include "config.h"
 #include "confile.h"
 #include "log.h"
-#include "rexec.h"
 #include "utils.h"
 
 lxc_log_define(lxc_attach, lxc);
 
-/**
- * This function will copy any binary that calls liblxc into a memory file and
- * will use the memfd to rexecute the binary. This is done to prevent attacks
- * through the /proc/self/exe symlink to corrupt the host binary when host and
- * container are in the same user namespace or have set up an identity id
- * mapping: CVE-2019-5736.
- */
-#ifdef ENFORCE_MEMFD_REXEC
-__attribute__((constructor)) static void lxc_attach_rexec(void)
-{
-	if (!getenv("LXC_MEMFD_REXEC") && lxc_rexec("lxc-attach")) {
-		fprintf(stderr, "Failed to re-execute lxc-attach via memory file descriptor\n");
-		_exit(EXIT_FAILURE);
-	}
-}
-#endif
-
 static int my_parser(struct lxc_arguments *args, int c, char *arg);
 static int add_to_simple_array(char ***array, ssize_t *capacity, char *value);
 static bool stdfd_is_pty(void);
@@ -93,6 +75,8 @@ static const struct option my_longopts[] = {
 	{"set-var", required_argument, 0, 'v'},
 	{"pty-log", required_argument, 0, 'L'},
 	{"rcfile", required_argument, 0, 'f'},
+	{"uid", required_argument, 0, 'u'},
+	{"gid", required_argument, 0, 'g'},
 	LXC_COMMON_OPTIONS
 };
 
@@ -143,6 +127,8 @@ Options :\n\
                     multiple times.\n\
   -f, --rcfile=FILE\n\
                     Load configuration file FILE\n\
+  -u, --uid=UID     Execute COMMAND with UID inside the container\n\
+  -g, --gid=GID     Execute COMMAND with GID inside the container\n\
 ",
 	.options      = my_longopts,
 	.parser       = my_parser,
@@ -208,6 +194,14 @@ static int my_parser(struct lxc_arguments *args, int c, char *arg)
 	case 'f':
 		args->rcfile = arg;
 		break;
+	case 'u':
+		if (lxc_safe_uint(arg, &args->uid) < 0)
+			return -1;
+		break;
+	case 'g':
+		if (lxc_safe_uint(arg, &args->gid) < 0)
+			return -1;
+		break;
 	}
 
 	return 0;
@@ -354,23 +348,25 @@ int main(int argc, char *argv[])
 			goto out;
 	}
 
-	if (command.program) {
-		ret = c->attach_run_wait(c, &attach_options, command.program,
-					 (const char **)command.argv);
-		if (ret < 0)
-			goto out;
-	} else {
+	if (my_args.uid)
+		attach_options.uid = my_args.uid;
+
+	if (my_args.gid)
+		attach_options.gid = my_args.gid;
+
+	if (command.program)
+		ret = c->attach(c, lxc_attach_run_command, &command, &attach_options, &pid);
+	else
 		ret = c->attach(c, lxc_attach_run_shell, NULL, &attach_options, &pid);
-		if (ret < 0)
-			goto out;
+	if (ret < 0)
+		goto out;
 
-		ret = lxc_wait_for_pid_status(pid);
-		if (ret < 0)
-			goto out;
+	ret = lxc_wait_for_pid_status(pid);
+	if (ret < 0)
+		goto out;
 
-		if (WIFEXITED(ret))
-			wexit = WEXITSTATUS(ret);
-	}
+	if (WIFEXITED(ret))
+		wexit = WEXITSTATUS(ret);
 
 out:
 	lxc_container_put(c);
