@@ -49,7 +49,6 @@
 #include "config.h"
 #include "log.h"
 #include "lxclock.h"
-#include "memory_utils.h"
 #include "namespace.h"
 #include "parse.h"
 #include "raw_syscalls.h"
@@ -330,30 +329,17 @@ again:
 	return status;
 }
 
-#ifdef HAVE_OPENSSL
-#include <openssl/evp.h>
+#if HAVE_LIBGNUTLS
+#include <gnutls/gnutls.h>
+#include <gnutls/crypto.h>
 
-static int do_sha1_hash(const char *buf, int buflen, unsigned char *md_value, int *md_len)
+__attribute__((constructor))
+static void gnutls_lxc_init(void)
 {
-	EVP_MD_CTX *mdctx;
-	const EVP_MD *md;
-
-	md = EVP_get_digestbyname("sha1");
-	if(!md) {
-		printf("Unknown message digest: sha1\n");
-		return -1;
-	}
-
-	mdctx = EVP_MD_CTX_new();
-	EVP_DigestInit_ex(mdctx, md, NULL);
-	EVP_DigestUpdate(mdctx, buf, buflen);
-	EVP_DigestFinal_ex(mdctx, md_value, md_len);
-	EVP_MD_CTX_free(mdctx);
-
-	return 0;
+	gnutls_global_init();
 }
 
-int sha1sum_file(char *fnam, unsigned char *digest, int *md_len)
+int sha1sum_file(char *fnam, unsigned char *digest)
 {
 	char *buf;
 	int ret;
@@ -407,7 +393,7 @@ int sha1sum_file(char *fnam, unsigned char *digest, int *md_len)
 	}
 
 	buf[flen] = '\0';
-	ret = do_sha1_hash(buf, flen, (void *)digest, md_len);
+	ret = gnutls_hash_fast(GNUTLS_DIG_SHA1, buf, flen, (void *)digest);
 	free(buf);
 	return ret;
 }
@@ -461,12 +447,7 @@ struct lxc_popen_FILE *lxc_popen(const char *command)
 		if (ret < 0)
 			_exit(EXIT_FAILURE);
 
-		/* check if /bin/sh exist, otherwise try Android location /system/bin/sh */
-		if (file_exists("/bin/sh"))
-			execl("/bin/sh", "sh", "-c", command, (char *)NULL);
-		else
-			execl("/system/bin/sh", "sh", "-c", command, (char *)NULL);
-
+		execl("/bin/sh", "sh", "-c", command, (char *)NULL);
 		_exit(127);
 	}
 
@@ -653,19 +634,12 @@ uint64_t fnv_64a_buf(void *buf, size_t len, uint64_t hval)
 	return hval;
 }
 
-/*
- * Detect whether / is mounted MS_SHARED.  The only way I know of to
- * check that is through /proc/self/mountinfo.
- * I'm only checking for /.  If the container rootfs or mount location
- * is MS_SHARED, but not '/', then you're out of luck - figuring that
- * out would be too much work to be worth it.
- */
-int detect_shared_rootfs(void)
+bool is_shared_mountpoint(const char *path)
 {
-	char buf[LXC_LINELEN], *p;
+	char buf[LXC_LINELEN];
 	FILE *f;
 	int i;
-	char *p2;
+	char *p, *p2;
 
 	f = fopen("/proc/self/mountinfo", "r");
 	if (!f)
@@ -682,34 +656,46 @@ int detect_shared_rootfs(void)
 			continue;
 
 		*p2 = '\0';
-		if (strcmp(p + 1, "/") == 0) {
-			/* This is '/'. Is it shared? */
+		if (strcmp(p + 1, path) == 0) {
+			/* This is the path. Is it shared? */
 			p = strchr(p2 + 1, ' ');
 			if (p && strstr(p, "shared:")) {
 				fclose(f);
-				return 1;
+				return true;
 			}
 		}
 	}
 
 	fclose(f);
+	return false;
+}
+
+/*
+ * Detect whether / is mounted MS_SHARED.  The only way I know of to
+ * check that is through /proc/self/mountinfo.
+ * I'm only checking for /.  If the container rootfs or mount location
+ * is MS_SHARED, but not '/', then you're out of luck - figuring that
+ * out would be too much work to be worth it.
+ */
+int detect_shared_rootfs(void)
+{
+	if (is_shared_mountpoint("/"))
+		return 1;
+
 	return 0;
 }
 
 bool switch_to_ns(pid_t pid, const char *ns)
 {
-	__do_close_prot_errno int fd = -EBADF;
-	int ret;
-	char nspath[STRLITERALLEN("/proc//ns/")
-		    + INTTYPE_TO_STRLEN(pid_t)
-		    + LXC_NAMESPACE_NAME_MAX];
+	int fd, ret;
+	char nspath[PATH_MAX];
 
 	/* Switch to new ns */
-	ret = snprintf(nspath, sizeof(nspath), "/proc/%d/ns/%s", pid, ns);
-	if (ret < 0 || ret >= sizeof(nspath))
+	ret = snprintf(nspath, PATH_MAX, "/proc/%d/ns/%s", pid, ns);
+	if (ret < 0 || ret >= PATH_MAX)
 		return false;
 
-	fd = open(nspath, O_RDONLY | O_CLOEXEC);
+	fd = open(nspath, O_RDONLY);
 	if (fd < 0) {
 		SYSERROR("Failed to open \"%s\"", nspath);
 		return false;
@@ -717,11 +703,12 @@ bool switch_to_ns(pid_t pid, const char *ns)
 
 	ret = setns(fd, 0);
 	if (ret) {
-		SYSERROR("Failed to set process %d to \"%s\" of %d.", pid, ns,
-			 fd);
+		SYSERROR("Failed to set process %d to \"%s\" of %d.", pid, ns, fd);
+		close(fd);
 		return false;
 	}
 
+	close(fd);
 	return true;
 }
 
@@ -918,13 +905,9 @@ char *get_template_path(const char *t)
 	int ret, len;
 	char *tpath;
 
-	if (t[0] == '/') {
-		if (access(t, X_OK) == 0) {
-			return strdup(t);
-		} else {
-			SYSERROR("Bad template pathname: %s", t);
-			return NULL;
-		}
+	if (t[0] == '/' && access(t, X_OK) == 0) {
+		tpath = strdup(t);
+		return tpath;
 	}
 
 	len = strlen(LXCTEMPLATEDIR) + strlen(t) + strlen("/lxc-") + 1;
@@ -1109,7 +1092,7 @@ static int open_without_symlink(const char *target, const char *prefix_skip)
 		goto out;
 	}
 
-	for (;;) {
+	while (1) {
 		int newfd, saved_errno;
 		char *nextpath;
 
@@ -1502,16 +1485,8 @@ static int lxc_get_unused_loop_dev(char *name_loop)
 		goto on_error;
 
 	fd_tmp = open(name_loop, O_RDWR | O_CLOEXEC);
-	if (fd_tmp < 0) {
-		/* on Android loop devices are moved under /dev/block, give it a shot */
-		ret = snprintf(name_loop, LO_NAME_SIZE, "/dev/block/loop%d", loop_nr);
-                if (ret < 0 || ret >= LO_NAME_SIZE)
-                        goto on_error;
-
-		fd_tmp = open(name_loop, O_RDWR | O_CLOEXEC);
-		if (fd_tmp < 0)
-			SYSERROR("Failed to open loop \"%s\"", name_loop);
-	}
+	if (fd_tmp < 0)
+		SYSERROR("Failed to open loop \"%s\"", name_loop);
 
 on_error:
 	close(fd_ctl);
@@ -1601,7 +1576,7 @@ pop_stack:
 	return umounts;
 }
 
-int run_command_internal(char *buf, size_t buf_size, int (*child_fn)(void *), void *args, bool wait_status)
+int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
 {
 	pid_t child;
 	int ret, fret, pipefd[2];
@@ -1616,7 +1591,7 @@ int run_command_internal(char *buf, size_t buf_size, int (*child_fn)(void *), vo
 		return -1;
 	}
 
-	child = lxc_raw_clone(0, NULL);
+	child = lxc_raw_clone(0);
 	if (child < 0) {
 		close(pipefd[0]);
 		close(pipefd[1]);
@@ -1658,27 +1633,13 @@ int run_command_internal(char *buf, size_t buf_size, int (*child_fn)(void *), vo
 			buf[bytes - 1] = '\0';
 	}
 
-	if (wait_status)
-		fret = lxc_wait_for_pid_status(child);
-	else
-		fret = wait_for_pid(child);
-
+	fret = wait_for_pid(child);
 	/* close the read-end of the pipe */
 	close(pipefd[0]);
 
 	return fret;
 }
 
-int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
-{
-    return run_command_internal(buf, buf_size, child_fn, args, false);
-}
-
-int run_command_status(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
-{
-    return run_command_internal(buf, buf_size, child_fn, args, true);
-}
-
 bool lxc_nic_exists(char *nic)
 {
 #define __LXC_SYS_CLASS_NET_LEN 15 + IFNAMSIZ + 1
@@ -1726,9 +1687,9 @@ int lxc_set_death_signal(int signal, pid_t parent)
 	ret = prctl(PR_SET_PDEATHSIG, prctl_arg(signal), prctl_arg(0),
 		    prctl_arg(0), prctl_arg(0));
 
-	/* If not in a PID namespace, check whether we have been orphaned. */
+	/* Check whether we have been orphaned. */
 	ppid = (pid_t)syscall(SYS_getppid);
-	if (ppid && ppid != parent) {
+	if (ppid != parent) {
 		ret = raise(SIGKILL);
 		if (ret < 0)
 			return -1;
