@@ -184,7 +184,7 @@ int main()
 	 * pam_cgroup */
 	if (geteuid() != 0) {
 		TSTERR("requires privilege");
-		exit(EXIT_SUCCESS);
+		exit(0);
 	}
 
 	#if TEST_ALREADY_RUNNING_CT
