@@ -767,6 +767,16 @@ int main(int argc, char *argv[])
 		goto non_test_error;
 	}
 
+	ret = set_get_compare_clear_save_load(c, "lxc.monitor.signal.pdeath", "SIGKILL", tmpf, true);
+	if (ret == 0) {
+		lxc_error("%s\n", "lxc.hook.version");
+		goto non_test_error;
+	}
+
+	if (set_get_compare_clear_save_load(c, "lxc.rootfs.managed", "1", tmpf, true) < 0) {
+		lxc_error("%s\n", "lxc.rootfs.managed");
+		goto non_test_error;
+	}
 
 	if (c->set_config_item(c, "lxc.notaconfigkey", "invalid")) {
 		lxc_error("%s\n", "Managed to set to set invalid config item \"lxc.notaconfigkey\" to \"invalid\"");
